<!--
 * @Author: Jerry-Rain
 * @Date: 2020-03-24 13:48:00
 * @LastEditors: Jerry-Rain
 * @LastEditTime: 2020-04-23 17:00:22
 * @Description: 新电量统计页面
 -->
<template>
  <div>
    <!-- 按钮组 -->
    <div class="button-group">
      <div class="btn-mask">
        <div
          v-for="(item, index) in typeList"
          :key="index"
          class="btn"
          @click="btnIndex(index)"
          :style="{
            color: '#162c54',
            background: index !== selcetedIndex ? '#dde0e1' : 'white'
          }"
        >
          {{ item }}
        </div>
      </div>
    </div>
    <!-- 图表区 -->
    <div class="loding" v-show="isLoading1 || isLoading2">
      <gree-activity-indicator class="loding-icon" type="spinner" :size="40"></gree-activity-indicator>
    </div>
    <div class="chart-bg" v-show="!isLoading1 && !isLoading2">
      <div class="tips-text" v-show="!isNoData">
        <p class="offer-text">{{ textList[selcetedIndex] + '发电量: ' + this.PVtotalType(selcetedIndex) + '度' }}</p>
        <p class="use-text">{{ textList[selcetedIndex] + '用电量: ' + this.totalType(selcetedIndex) + '度' }}</p>
      </div>
      <gree-block class="chart-block" v-show="!isNoData">
        <div class="chart-canves" ref="echart" />
      </gree-block>
      <div v-show="isNoData" class="isNoData">
        <p class="no-data-text" v-text="'当前无用电信息'" />
      </div>
      <gree-row v-show="!isNoData" class="tip">以上数据仅供参考</gree-row>
    </div>
    <!-- 清除电量按钮 -->
    <gree-button class="btn-cln-elc" type="primary" :disabled="btnDisable" @click="cleanElc" block round>电量清零</gree-button>
  </div>
</template>

<script>
import echarts from 'echarts';
import { mapState, mapMutations, mapActions } from 'vuex';
import chartsConfig from '@/mixins/PVecharts';
import { types } from '@/store/types';
import { Header, Row, Col, Block, Divider, Dialog, ToolBar, Button, ActivityIndicator } from 'gree-ui';
import { showToast, getGridConList, getElecGenerList, clearHistoricalPhotovoltaicPowerData } from '@PluginInterface';

export default {
  name: 'PVElectricChart',
  components: {
    [Row.name]: Row,
    [Col.name]: Col,
    [Block.name]: Block,
    [Dialog.name]: Dialog,
    [Button.name]: Button,
    [Header.name]: Header,
    [Divider.name]: Divider,
    [ToolBar.name]: ToolBar,
    [ActivityIndicator.name]: ActivityIndicator
  },
  mixins: [chartsConfig],
  data() {
    return {
      DEBUG: false, // 测试
      typeList: ['日报', '月报', '年报', '历年'],
      textList: ['今日累计', '当月累计', '今年累计', '累计总'],
      selcetedIndex: 0,
      echart: '',
      isNoData: true, // 无数据
      tooltip: {},
      dataArr: [],
      timeArr: [],
      isLoading1: true,
      isLoading2: true
    };
  },
  computed: {
    ...mapState('control', {
      mac: state => state.mac,
      isClear: state => state.isClear,
      chartData: state => state.chartData,
      dayTotal: state => state.chartData.dayTotal,
      monthTotal: state => state.chartData.monthTotal || 0,
      yearTotal: state => state.chartData.yearTotal || 0,
      allTotal: state => state.chartData.allTotal || 0,
      PVdayTotal: state => state.chartData.PVdayTotal,
      PVmonthTotal: state => state.chartData.PVmonthTotal || 0,
      PVyearTotal: state => state.chartData.PVyearTotal || 0,
      PVallTotal: state => state.chartData.PVallTotal || 0
    }),
    btnDisable() {
      return this.isNoData || (this.isLoading1 && this.isLoading2);
    }
  },
  mounted() {
    if (this.DEBUG) {
      // 调试模式时进入页面手动赋值
      this.setChartData(this.PVdebugData);
      this.chartSet();
      this.refreshChart(this.chartData.dayChart[0], this.chartData.dayChart[1], this.chartData.PVdayChart[0], this.chartData.PVdayChart[1]);
    } else {
      this.getDataByIndex(0);
    }
    this.initCharts(); // 初始化图表
  },
  beforeDestroy() {
    this.setChartData({ dayChart: null, PVdayChart: null });
  },
  methods: {
    ...mapMutations('control', {
      setChartData: 'SET_CHART_DATA'
    }),
    ...mapActions({
      sendCtrl: types.SEND_CTRL
    }),
    // 初始化图表
    initCharts() {
      this.echart = echarts.init(this.$refs.echart);
    },
    // 当前选择类型请求对应的数据
    btnIndex(index) {
      this.isLoading1 = true;
      this.isLoading2 = true;
      this.echart.clear();
      this.echart.resize();
      this.DEBUG ? this.setChartData(this.PVdebugData) : ''; // 测试数据
      this.selcetedIndex = index;
      this.getDataByIndex(index);
      this.DEBUG && this.deBugFunc(index);
    },
    // 调试模式下切换tab时手动赋值
    deBugFunc(index) {
      const refreshType = ['dayChart', 'weekChart', 'monthChart', 'yearChart'];
      const PVrefreshType = ['PVdayChart', 'PVweekChart', 'PVmonthChart', 'PVyearChart'];
      this.setChartData(this.PVdebugData);
      this.chartSet();
      this.refreshChart(
        this.chartData[refreshType[index]][0],
        this.chartData[refreshType[index]][1],
        this.chartData[PVrefreshType[index]][0],
        this.chartData[PVrefreshType[index]][1]
      );
    },
    totalType(selcetedIndex) {
      const totalList = [this.dayTotal, this.monthTotal, this.yearTotal, this.allTotal];
      const result = totalList[selcetedIndex];
      return result;
    },
    PVtotalType(selcetedIndex) {
      const PVtotalList = [this.PVdayTotal, this.PVmonthTotal, this.PVyearTotal, this.PVallTotal];
      const result = PVtotalList[selcetedIndex];
      return result;
    },
    // 清除电量dialog
    cleanElc() {
      Dialog.confirm({
        title: '提示',
        message: '电量清零后不可恢复，是否确定清零？',
        confirmButtonText: '确定',
        cancelButtonText: '取消'
      })
        .then(() => {
          this.cleanElcInterface();
        })
        .catch(() => {
          console.log('cancel');
        });
    },
    setValueByIndex(index) {
      const parameterList = ['day', 'month', 'year', 'all'];
      const result = parameterList[index];
      return result;
    },
    chooseFuncByIndex(index, type) {
      index === 0 && this.getDayData(type);
      index === 1 && this.getMonthData(type);
      index === 2 && this.getYearData(type);
      index === 3 && this.getAllData(type);
    },
    getDayData(type) {
      const totalType = ['dayTotal', 'PVdayTotal'];
      const chartType = ['dayChart', 'PVdayChart'];
      let obj = {};
      const time = [];
      let realValue = 0;
      let realValueArr = [];
      let value = new Array(12).fill(0);
      const arr1 = Object.values(this.dataArr);
      const arr2 = Object.values(this.timeArr);
      arr1.forEach((v, i) => {
        realValue = v - arr1[i - 1];
        realValueArr.push(realValue);
        realValueArr[0] = ~~realValueArr[0];
      });
      //  日求和
      if (this.dataArr.length > 0) {
        let result = this.dataArr[this.dataArr.length - 1];
        this.setChartData({ [totalType[type]]: Math.round(result * 1000) / 1000 });
      }
      for (let i = 0; i < this.timeArr.length; i++) {
        const objKey = this.timeArr[i];
        const objValue = realValueArr[i];
        obj[objKey] = objValue;
      }
      arr2.forEach(v => {
        value[v / 2] = obj[v];
      });
      for (let i = 0; i < 12; i += 1) {
        time[i] = 2 * i + ':00';
      }
      this.setChartData({ [chartType[type]]: [value, time] });
    },
    getMonthData(type) {
      try {
        const chartType = ['weekChart', 'PVweekChart'];
        const totalType = ['monthTotal', 'PVmonthTotal'];
        const d = new Date(new Date().getFullYear(), new Date().getMonth() + 1, 0);
        const monthCount = d.getDate();
        const month = [];
        const count = [];
        let result = 0;
        for (let i = 0; i < monthCount; i += 1) {
          month[i] = 0;
          count[i] = i + 1;
        }
        for (let i = 0; i < this.timeArr.length; i += 1) {
          month[this.timeArr[i] - 1] = this.dataArr[i];
        }
        console.log(123456, this.dataArr);
        //  月求和
        if (this.dataArr.length > 0) {
          for (let i = 0; i < this.dataArr.length; i++) {
            result += this.dataArr[i];
          }
          this.setChartData({ [totalType[type]]: Math.round(result * 1000) / 1000 });
        }
        this.setChartData({ [chartType[type]]: [month, count] });
      } catch (error) {
        console.log('报错');
      }
    },
    getYearData(type) {
      try {
        const chartType = ['monthChart', 'PVmonthChart'];
        const totalType = ['yearTotal', 'PVyearTotal'];
        const year = [];
        const count = [];
        let result = 0;
        for (let i = 0; i < 12; i += 1) {
          year[i] = 0;
          count[i] = i + 1;
        }
        for (let i = 0; i < this.timeArr.length; i += 1) {
          year[this.timeArr[i] - 1] = this.dataArr[i];
        }
        //  年求和
        if (this.dataArr.length > 0) {
          for (let i = 0; i < this.dataArr.length; i++) {
            result += this.dataArr[i];
          }
          this.setChartData({ [totalType[type]]: Math.round(result * 1000) / 1000 });
        }
        this.setChartData({ [chartType[type]]: [year, count] });
      } catch (error) {
        console.log('报错');
      }
    },
    getAllData(type) {
      try {
        const chartType = ['yearChart', 'PVyearChart'];
        const totalType = ['allTotal', 'PVallTotal'];
        const all = [];
        const count = [];
        let result = 0;
        for (let i = 0; i < this.timeArr.length; i += 1) {
          all[i] = this.dataArr[i];
          count[i] = this.timeArr[i];
        }
        //  历年求和
        if (this.dataArr.length > 0) {
          for (let i = 0; i < this.dataArr.length; i++) {
            result += this.dataArr[i];
          }
          this.setChartData({ [totalType[type]]: Math.round(result * 1000) / 1000 });
        }
        this.setChartData({ [chartType[type]]: [all, count] });
      } catch (error) {
        console.log('报错');
      }
    },
    onRefreshChart(key) {
      if (key === 'day') {
        this.refreshChart(this.chartData.dayChart[0], this.chartData.dayChart[1], this.chartData.PVdayChart[0], this.chartData.PVdayChart[1]);
      }
      if (key === 'month') {
        this.refreshChart(this.chartData.weekChart[0], this.chartData.weekChart[1], this.chartData.PVweekChart[0], this.chartData.PVweekChart[1]);
      }
      if (key === 'year') {
        this.refreshChart(this.chartData.monthChart[0], this.chartData.monthChart[1], this.chartData.PVmonthChart[0], this.chartData.PVmonthChart[1]);
      }
      if (key === 'all') {
        this.refreshChart(this.chartData.yearChart[0], this.chartData.yearChart[1], this.chartData.PVyearChart[0], this.chartData.PVyearChart[1]);
      }
    },
    async getDataByIndex(index) {
      const key = this.setValueByIndex(index);
      console.log('-xxxxx测试---------------', key);
      const data1 = await getGridConList(this.mac, key);
      const obj1 = JSON.parse(data1);
      if (obj1.r === 200) {
        const { val, time } = obj1.data;
        console.log(`${key}用电量-----`, obj1);
        this.dataArr = val;
        this.timeArr = time;
        this.isLoading1 = false;
        this.chooseFuncByIndex(index, 0);
      }
      const data2 = await getElecGenerList(this.mac, key);
      const obj2 = JSON.parse(data2);
      if (obj2.r === 200) {
        console.log(`${key}发电量-----`, obj2);
        const { val, time } = obj2.data;
        this.dataArr = val;
        this.timeArr = time;
        this.isLoading2 = false;
        this.chooseFuncByIndex(index, 1);
      }
      await Promise.all([data1, data2]).then(() => {
        this.chartSet();
        this.onRefreshChart(key);
      });
    },
    // 清除电量接口
    cleanElcInterface() {
      clearHistoricalPhotovoltaicPowerData(this.mac)
        .then(result => {
          const res = JSON.parse(result);
          // 判断是否成功清除电量
          if (res.r === 400) {
            console.log('Api调用失败');
          } else if (res.r === 200) {
            console.log('成功请求，清除电量');
            this.sendCtrl({ Electnumzero: 0 });
            this.chartsResize();
            window.$bus.$emit('claerPVData');
          } else if (res.r === 10700) {
            showToast('您没有权限清除电量哦', 0);
          } else {
            console.log('失败！其他原因');
          }
        })
        .catch(err => {
          console.log('err', err);
        });
    },
    chartsResize() {
      this.echart.clear();
      this.isNoData = true;
    },
    refreshChart(fristValue, fristCount, secondValue, secondCount) {
      console.log('refreshChart===', fristValue, fristCount, secondValue, secondCount);
      this.dayChartOpt.xAxis.data = fristCount;
      this.dayChartOpt.series[0].data = fristValue;
      this.dayChartOpt.series[1].data = secondValue;
      setTimeout(() => {
        this.echart.setOption(this.dayChartOpt, true);
        this.$nextTick(() => {
          this.echart.resize();
        });
      }, 0);
    },
    chartSet() {
      this.isLoading1 = false;
      this.isLoading2 = false;
      this.isNoData = false;
      this.dayChartOpt.grid.show = false;
      this.dayChartOpt.xAxis.axisLabel.interval = 3;
      this.dayChartOpt.grid = { left: '6%', top: '30%', right: '6%', bottom: '10%', containLabel: true };
      this.dayChartOpt.tooltip.formatter = params => {
        let res = '';
        const unitList = ['日', '月', '年'];
        const nowTime = params[0].axisValue;
        const generatedEnergy = params[1].data;
        const ElectricityConsumption = params[0].data;
        if (this.selcetedIndex === 0) {
          const lastTime = nowTime.substr(0, nowTime.indexOf(':')) - 2 + ':00';
          if (nowTime === '0:00' || nowTime === '00:00') {
            res += '0:00';
          } else {
            res += `${lastTime + '-' + nowTime}`;
          }
        } else {
          res += nowTime + unitList[this.selcetedIndex - 1];
        }
        res += `<p><label style="color:#ffffff;">发电量</label>: ${Math.round(generatedEnergy * 1000) / 1000 + '度'}</p>`;
        res += `<p><label style="color:#ffffff;">用电量</label>: ${Math.round(ElectricityConsumption * 1000) / 1000 + '度'}</p>`;
        return res;
      };
    },
    goBack() {
      this.$router.go(-1);
    }
  }
};
</script>
